#ifndef _GALILAXIS_H
#define _GALILAXIS_H

#include <tango.h>
// Add your own constants definitions here.
//-----------------------------------------------
const int AXIS_MOTION_STATUS_REFRESH_TIME=500; //msec


namespace GalilAxis_ns
{

/*
 *	Device States Description:
 *	Tango::OFF :	Communication broken with the motion controller
 *	Tango::STANDBY :	Axis OK, waiting for commands
 *	Tango::MOVING :	Axis is moving
 *	Tango::FAULT :	Failure on the axis. Can be hardware failure .
 *	Tango::ALARM :	Alarm on the axis. can be soft or hard limit, abort input...
 *	Tango::DISABLE :	For debug purpose
 */


	class GalilAxis: public Tango::Device_4Impl
	{
	public :
		//	Add your own data members here
		//-----------------------------------------
		// current parameters, saved each write_attr_hardware, coded for galil in string		
		std::string GalilAcceleration;
		std::string GalilDeceleration;
		std::string GalilVelocity;
		std::string GalilPercent;
		std::string GalilAccuracy;
		std::string GalilRetry;
		std::string GalilEncoderMotorRatio;
		std::string GalilBacklash;
		std::string GalilDirection;
		
		
		bool AxisCurrentEncoderPositionQuality;
		
	private:
		Tango::DeviceProxy* galilbox_device_proxy;
		
	private:
		
		virtual void delete_device();
		
		// routines de conversion User Unit/Encoder Unit
		// With Offset Management for positions
		
		Tango::DevDouble	encoder_units_to_physical_units(Tango::DevLong);
		Tango::DevLong		physical_units_to_encoder_units(Tango::DevDouble);
		// Without Offset Management for speed, acceleration
		Tango::DevDouble	encoder_units_to_user_units(Tango::DevLong);
		Tango::DevLong		user_units_to_encoder_units(Tango::DevDouble);
		Tango::DevLong moving_is_greater_than_a_step(Tango::DevDouble);
		
		// methode de gestion de la communication avec le GalilBox
		std::string axis_command(const std::string &argin );
		// Method to store an attribute value as a property in database
		void store_value_as_property(Tango::DevDouble value, string property_name);
		/*
		Method to set the Write Value of an Attribute
		returns the index of the	WAttribute object in the 
		*/
		int set_write_value(Tango::DevDouble value, string property_name);
		int set_write_value(Tango::DevLong value, string property_name);
		
		void log_exception(Tango::DevFailed&	e );
		
	public:
		
		//	Here is the Start of the automatic code generation part
		//-------------------------------------------------------------
		/**
		*	@name attributes
		*	Attributs member data.
		*/
		//@{
		Tango::DevDouble	*attr_AxisCurrentPosition_read;
		Tango::DevDouble	attr_AxisCurrentPosition_write;
		Tango::DevLong	*attr_AxisCurrentEncoderPosition_read;
		Tango::DevLong	*attr_AxisCurrentMotorPosition_read;
		Tango::DevDouble	*attr_AxisAcceleration_read;
		Tango::DevDouble	attr_AxisAcceleration_write;
		Tango::DevDouble	*attr_AxisDeceleration_read;
		Tango::DevDouble	attr_AxisDeceleration_write;
		Tango::DevDouble	*attr_AxisVelocity_read;
		Tango::DevDouble	attr_AxisVelocity_write;
		Tango::DevDouble	*attr_AxisMotionAccuracy_read;
		Tango::DevDouble	attr_AxisMotionAccuracy_write;
		Tango::DevDouble	*attr_AxisPercent_read;
		Tango::DevDouble	attr_AxisPercent_write;
		Tango::DevDouble	*attr_AxisRetry_read;
		Tango::DevDouble	attr_AxisRetry_write;
		Tango::DevDouble	*attr_AxisBacklash_read;
		Tango::DevDouble	attr_AxisBacklash_write;
		Tango::DevDouble	*attr_AxisDirection_read;
		Tango::DevDouble	attr_AxisDirection_write;
		Tango::DevDouble	*attr_AxisOffset_read;
		Tango::DevDouble	attr_AxisOffset_write;
		Tango::DevShort	*attr_UserDirection_read;
		Tango::DevShort	attr_UserDirection_write;
		Tango::DevDouble	*attr_AxisCurrentVelocity_read;
		Tango::DevShort	*attr_IsInitialised_read;
		Tango::DevShort	*attr_LatchMode_read;
		Tango::DevShort	attr_LatchMode_write;
		Tango::DevDouble	*attr_PositionLatched_read;
		Tango::DevShort	*attr_LatchOccured_read;
//@}
		
		
		/**
		*	@name Device properties
		*	Device properties member data.
		*/
		//@{
/**
 *	Number of the Axis on the motion controller (also called GalilBox).
 *	Depending on the GalilBox model it may be from 1 to 8
 *	OR ( new ) A to H (Galil terminology )
 *	1=A =X 2=B=Y 3 =C=Z 4=D=W 5=E 6=F 7=G 8=H
 */
	string	axisNumber;
/**
 *	TANGO device name of the Galil motion controller ( also called GalilBox )
 *	which drives this axis <Br>
 *	<I> Example: </I> <B> microxas/motorisation/galilbox3 </B>
 */
	string	axisBoxAttachement;
/**
 *	Number of Encoder steps for 1 Motor step. <BR>
 *	
 *	<I> Example: </I> If <B>EncoderMotorRatio=4 </B> , for a 1000 motor steps movement,
 *	Encoder value will change of 4000 steps
 *	
 *	<BR>
 *	When no Encoder is available for the Axis , this parameter must be set to 1. <BR>
 */
	Tango::DevDouble	encoderMotorRatio;
/**
 *	Property used to define the conversion between <I>EncoderUnits</I> to <I>UserUnits</I> <BR>
 *	according to the following formula: <BR>
 *	
 *	<B> AxisCurrentPosition (<I> in UserUnits </I>) = AxisEncoderPosition
 *	UserEncoderRatio + CurrentOffset
 *	</B>
 *	
 *	<BR>
 *	<B> Exemple </B> :
 *	<UL>
 *	<LI> UserEncoderRatio = 0.1</LI>
 *	<LI> CurrentOffset = 33</LI>
 *	<LI> AxisEncoderPosition = 100 <I> EncoderUnits </I></LI>
 *	<LI> then AxisCurrentPosition=
 *	43 <I>UserUnits <I> </LI>
 *	</UL>
 */
	Tango::DevDouble	userEncoderRatio;
/**
 *	Property used to define the conversion between <I>EncoderUnits</I> to <I>UserUnits</I> <BR>
 *	according to the following formula: <BR>
 *	
 *	<B> AxisCurrentPosition (<I> in UserUnits </I>) = AxisEncoderPosition
 *	UserEncoderRatio + CurrentOffset
 *	</B>
 *	
 *	<BR>
 *	<B> Exemple </B> :
 *	<UL>
 *	<LI> UserEncoderRatio = 0.1</LI>
 *	<LI> CurrentOffset = 33</LI>
 *	<LI> AxisEncoderPosition = 100 <I> EncoderUnits </I></LI>
 *	<LI> then AxisCurrentPosition=
 *	43 <I>UserUnits <I> </LI>
 *	</UL>
 */
	Tango::DevDouble	currentOffset;
/**
 *	Property used to store the current AxisAcceleration value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentAcceleration;
/**
 *	Property used to store the current AxisDeceleration value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentDeceleration;
/**
 *	Property used to store the current AxisVelocity value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentVelocity;
/**
 *	Property used to store the current AxisPercent value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentPercent;
/**
 *	Property used to store the current AxisAccuracy value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentMotionAccuracy;
/**
 *	Property used to store the current AxisRetry value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentRetry;
/**
 *	Type of Encoder connected to the axis:
 *	<UL>
 *	<LI> 0 for no encoder </LI>
 *	<LI> 1 for incremental encoder </LI>
 *	<LI> 2 for absolute encoder </LI>
 *	</UL>
 */
	Tango::DevLong	axisEncoderType;
/**
 *	Property used to store the current AxisBacklash value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentBacklash;
/**
 *	Property used to store the current AxisDirection value in TANGO DataBase.
 *	Not to be configured by user
 */
	Tango::DevDouble	currentDirection;
/**
 *	Use this property to smooth acceleration/deceleration.
 *	Usefull for stepper motors drived in full step.
 *	range : 0.5 ( no smoothing ) to 8.0 (max smoothing ) per steps of 1/32
 *	Default : 0.5
 *	See Galil command reference KS
 */
	Tango::DevDouble	axisSmoothing;
/**
 *	used to have uniform positive direction for motors
 *	rotate : positive is trigonometric direction
 *	translate : positive is UP or ....?
 *	This applies to all movements ( AxisForward, AxisGoToPosition....)
 *	1 : servo motor
 *	-1 :servo motor with reverse polarity
 *	2 : stepper motor with active low step pulse
 *	-2: stepper motor with active high step pulse
 *	2.5 :stepper motor with reverse direction and active low step pulse
 *	-2.5 :stepper motor with reverse direction and active high step pulse
 *	Default : 2
 *	See Galil command reference MT
 */
	Tango::DevDouble	axisMotorDirection;
/**
 *	ONLY FOR INCREMENTAL ENCODERS !
 *	configures the encoder type and direction
 *	Applies to main and aux encoders.
 *	NOTE : for stepper motor operation ONLY USE THE MAIN ENCODER!
 *	____________________________________________________________________
 *	m= Main encoder type n= Aux encoder type
 *	-------------------------------------------------------------------------------------------------------------------------
 *	0 Normal quadrature 0 Normal quadrature
 *	1 Normal pulse and direction 4 Normal pulse and direction
 *	2 Reversed quadrature 8 Reversed quadrature
 *	3 Reversed pulse and direction 12 Reversed pulse and direction
 *	default : 0
 *	Do the sum of m and n
 *	
 *	See Galil commmand reference CE
 */
	Tango::DevShort	axisEncoderDirection;
/**
 *	This is the Motion controller software forward limit
 *	Stops any movement passing that limit
 *	
 *	default : 1000000
 *	See Galil command FL
 */
	Tango::DevLong	axisForwardLimit;
/**
 *	Motion controller software backward limit.
 *	default : -1000000
 *	see Galil command reference BL
 */
	Tango::DevLong	axisBackwardLimit;
/**
 *	used to store the UserDirection attribute
 *	not to be modified directly.
 */
	Tango::DevShort	currentUserDirection;
/**
 *	in encoder unit, adds the specified offset to the CurrentEncoderPosition
 *	Modifies the CurrentPosition
 *	Used only for absolute encoders.
 */
	Tango::DevLong	absoluteEncoderOffset;
/**
 *	allows motorist to change the direction of increment position on ABSOLUTE ENCODERS
 *	must be 1 or -1
 *	Default 1
 */
	Tango::DevShort	absoluteEncoderDirection;
/**
 *	for initialisation of the axis.
 *	Not relevant for axis with absolute encoders or axis without limit switches and without encoders
 *	The axis will move regarding to its type and load the InitPosition property in the motion controller
 *	describes the type of init to be done regarding swiches and axis type
 *	0 : no initialisation possible
 *	1 : initialisation on HOME switch, uses the HM command of the motion controller
 *	2 : initialisation on LIMIT SWITCH BACKWARD (User Direction)
 *	4 : initialisation on LIMIT SWITCH FORWARD ( User Direction )
 *	3 : on TOP ZERO of the incremental encoder
 */
	Tango::DevShort	axisInitType;
/**
 *	position of the axis on its initialisation position
 *	see AxisInitType and Desrver documentation
 *	in user units
 */
	Tango::DevDouble	axisInitPosition;
/**
 *	Makes nothing. For some memos for the motorist.
 */
	string	motoristMemo;
/**
 *	For Servo motors only ( DC or bruhless motors )
 *	proportionall componement of the PID error loop correction
 *	Default : 6
 */
	Tango::DevFloat	servoPIDProp;
/**
 *	For Servo motors only ( DC or bruhsless motors )
 *	Integral componement of the PID error loop correction
 *	Default : 0
 */
	Tango::DevFloat	servoPIDInteg;
/**
 *	For Servo motors only ( DC or bruhsless motors )
 *	Derivate componement of the PID error loop correction
 *	Default : 64
 */
	Tango::DevFloat	servoPIDDeriv;
/**
 *	For Servo motors only ( DC or bruhsless motors )
 *	Limitation of the Integral componement of the PID error loop correction
 *	A negative parameter freezes the effect of the parameter during the move.
 *	Default : 5.0
 */
	Tango::DevFloat	servoPIDIntegLimit;
/**
 *	Error limit sets the magnitude of the dynamic position error ( during the move ).
 *	When the limit is exceeded, the motor is disabled.
 *	-1 inhibits error limit.
 *	Must never be reached under normal operation, only in case
 *	of hardware jam ( motor, encoder, mechanics )
 *	Default : -1
 *	Maximum : 32767
 */
	Tango::DevShort	errorLimit;
//@}
		
		/**@name Constructors
		* Miscellaneous constructors */
		//@{
		/**
		* Constructs a newly allocated Command object.
		*
		*	@param cl	Class.
		*	@param s		Device Name
		*/
		GalilAxis(Tango::DeviceClass *cl,string &s);
		/**
		* Constructs a newly allocated Command object.
		*
		*	@param cl	Class.
		*	@param s		Device Name
		*/
		GalilAxis(Tango::DeviceClass *cl,const char *s);
		/**
		* Constructs a newly allocated Command object.
		*
		*	@param cl	Class.
		*	@param s		Device name
		*	@param d		Device description.
		*/
		GalilAxis(Tango::DeviceClass *cl,const char *s,const char *d);
		//@}
		
		/**@name Destructor
		* Only one desctructor is defined for this class */
		//@{
		/**
		* The object desctructor.
		*/ 
		virtual ~GalilAxis() {delete_device();};
		//@}
		
		
		/**@name Miscellaneous methods */
		//@{
		/**
		*	Initialize the device
		*/
		virtual void init_device();
		/**
		*	Always executed method befor execution command method.
		*/
		virtual void always_executed_hook();

//@}

/**
 * @name GalilAxis methods prototypes
 */

//@{
/**
 *	Hardware acquisition for attributes.
 */
	virtual void read_attr_hardware(vector<long> &attr_list);
/**
 *	Extract real attribute values for AxisCurrentPosition acquisition result.
 */
	virtual void read_AxisCurrentPosition(Tango::Attribute &attr);
/**
 *	Write AxisCurrentPosition attribute values to hardware.
 */
	virtual void write_AxisCurrentPosition(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisCurrentEncoderPosition acquisition result.
 */
	virtual void read_AxisCurrentEncoderPosition(Tango::Attribute &attr);
/**
 *	Extract real attribute values for AxisCurrentMotorPosition acquisition result.
 */
	virtual void read_AxisCurrentMotorPosition(Tango::Attribute &attr);
/**
 *	Extract real attribute values for AxisAcceleration acquisition result.
 */
	virtual void read_AxisAcceleration(Tango::Attribute &attr);
/**
 *	Write AxisAcceleration attribute values to hardware.
 */
	virtual void write_AxisAcceleration(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisDeceleration acquisition result.
 */
	virtual void read_AxisDeceleration(Tango::Attribute &attr);
/**
 *	Write AxisDeceleration attribute values to hardware.
 */
	virtual void write_AxisDeceleration(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisVelocity acquisition result.
 */
	virtual void read_AxisVelocity(Tango::Attribute &attr);
/**
 *	Write AxisVelocity attribute values to hardware.
 */
	virtual void write_AxisVelocity(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisMotionAccuracy acquisition result.
 */
	virtual void read_AxisMotionAccuracy(Tango::Attribute &attr);
/**
 *	Write AxisMotionAccuracy attribute values to hardware.
 */
	virtual void write_AxisMotionAccuracy(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisPercent acquisition result.
 */
	virtual void read_AxisPercent(Tango::Attribute &attr);
/**
 *	Write AxisPercent attribute values to hardware.
 */
	virtual void write_AxisPercent(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisRetry acquisition result.
 */
	virtual void read_AxisRetry(Tango::Attribute &attr);
/**
 *	Write AxisRetry attribute values to hardware.
 */
	virtual void write_AxisRetry(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisBacklash acquisition result.
 */
	virtual void read_AxisBacklash(Tango::Attribute &attr);
/**
 *	Write AxisBacklash attribute values to hardware.
 */
	virtual void write_AxisBacklash(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisDirection acquisition result.
 */
	virtual void read_AxisDirection(Tango::Attribute &attr);
/**
 *	Write AxisDirection attribute values to hardware.
 */
	virtual void write_AxisDirection(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisOffset acquisition result.
 */
	virtual void read_AxisOffset(Tango::Attribute &attr);
/**
 *	Write AxisOffset attribute values to hardware.
 */
	virtual void write_AxisOffset(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for UserDirection acquisition result.
 */
	virtual void read_UserDirection(Tango::Attribute &attr);
/**
 *	Write UserDirection attribute values to hardware.
 */
	virtual void write_UserDirection(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for AxisCurrentVelocity acquisition result.
 */
	virtual void read_AxisCurrentVelocity(Tango::Attribute &attr);
/**
 *	Extract real attribute values for IsInitialised acquisition result.
 */
	virtual void read_IsInitialised(Tango::Attribute &attr);
/**
 *	Extract real attribute values for LatchMode acquisition result.
 */
	virtual void read_LatchMode(Tango::Attribute &attr);
/**
 *	Write LatchMode attribute values to hardware.
 */
	virtual void write_LatchMode(Tango::WAttribute &attr);
/**
 *	Extract real attribute values for PositionLatched acquisition result.
 */
	virtual void read_PositionLatched(Tango::Attribute &attr);
/**
 *	Extract real attribute values for LatchOccured acquisition result.
 */
	virtual void read_LatchOccured(Tango::Attribute &attr);
/**
 *	Read/Write allowed for AxisCurrentPosition attribute.
 */
	virtual bool is_AxisCurrentPosition_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisCurrentEncoderPosition attribute.
 */
	virtual bool is_AxisCurrentEncoderPosition_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisCurrentMotorPosition attribute.
 */
	virtual bool is_AxisCurrentMotorPosition_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisAcceleration attribute.
 */
	virtual bool is_AxisAcceleration_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisDeceleration attribute.
 */
	virtual bool is_AxisDeceleration_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisVelocity attribute.
 */
	virtual bool is_AxisVelocity_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisMotionAccuracy attribute.
 */
	virtual bool is_AxisMotionAccuracy_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisPercent attribute.
 */
	virtual bool is_AxisPercent_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisRetry attribute.
 */
	virtual bool is_AxisRetry_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisBacklash attribute.
 */
	virtual bool is_AxisBacklash_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisDirection attribute.
 */
	virtual bool is_AxisDirection_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisOffset attribute.
 */
	virtual bool is_AxisOffset_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for UserDirection attribute.
 */
	virtual bool is_UserDirection_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for AxisCurrentVelocity attribute.
 */
	virtual bool is_AxisCurrentVelocity_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for IsInitialised attribute.
 */
	virtual bool is_IsInitialised_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for LatchMode attribute.
 */
	virtual bool is_LatchMode_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for PositionLatched attribute.
 */
	virtual bool is_PositionLatched_allowed(Tango::AttReqType type);
/**
 *	Read/Write allowed for LatchOccured attribute.
 */
	virtual bool is_LatchOccured_allowed(Tango::AttReqType type);
/**
 *	Execution allowed for AxisForward command.
 */
	virtual bool is_AxisForward_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisStop command.
 */
	virtual bool is_AxisStop_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisBackward command.
 */
	virtual bool is_AxisBackward_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for StartMicroCode command.
 */
	virtual bool is_StartMicroCode_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisGoToRelativePosition command.
 */
	virtual bool is_AxisGoToRelativePosition_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisGoToPosition command.
 */
	virtual bool is_AxisGoToPosition_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisOscillate command.
 */
	virtual bool is_AxisOscillate_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisDefinePosition command.
 */
	virtual bool is_AxisDefinePosition_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisGetMotionStatus command.
 */
	virtual bool is_AxisGetMotionStatus_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisFindEdge command.
 */
	virtual bool is_AxisFindEdge_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisFindIndex command.
 */
	virtual bool is_AxisFindIndex_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisFindHome command.
 */
	virtual bool is_AxisFindHome_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisGetErrorStatus command.
 */
	virtual bool is_AxisGetErrorStatus_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for DirectCmd command.
 */
	virtual bool is_DirectCmd_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisInit command.
 */
	virtual bool is_AxisInit_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for MotorOFF command.
 */
	virtual bool is_MotorOFF_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for MotorON command.
 */
	virtual bool is_MotorON_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for AxisReadHWConf command.
 */
	virtual bool is_AxisReadHWConf_allowed(const CORBA::Any &any);
/**
 *	Execution allowed for GetUserEncoderRatio command.
 */
	virtual bool is_GetUserEncoderRatio_allowed(const CORBA::Any &any);
/**
 * This command gets the device state (stored in its <i>device_state</i> data member) and returns it to the caller.
 *	@return	State Code
 *	@exception DevFailed
 */
	virtual Tango::DevState	dev_state();
/**
 * This command gets the device status (stored in its <i>device_status</i> data member) and returns it to the caller.
 *	@return	Status descrition
 *	@exception DevFailed
 */
	virtual Tango::ConstDevString	dev_status();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_forward();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_stop();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_backward();
/**
 * 
 *	@param	argin	must be INIT reinit of all variables  or NOINIT just restart the microcode of GalilBox
 *	@exception DevFailed
 */
	void	start_micro_code(Tango::DevString);
/**
 * 
 *	@param	argin	Relative move in user unit
 *	@exception DevFailed
 */
	void	axis_go_to_relative_position(Tango::DevDouble);
/**
 * 
 *	@param	argin	Absolute position to reach in user units
 *	@exception DevFailed
 */
	void	axis_go_to_position(Tango::DevDouble);
/**
 * 
 *	@param	argin	Oscillation amplitude in user unit
 *	@exception DevFailed
 */
	void	axis_oscillate(Tango::DevDouble);
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevString	axis_define_position(Tango::DevDouble);
/**
 * Status of the motion : Returned Value is
 *	<UL>
 *	<LI> -1 motion state unknown because communication is broken </LI>
 *	<LI> 0 motion ended with success</LI>
 *	<LI> 1 positionning motion running</LI>
 *	</UL>
 *	@return	numerical code representing the status of the motion
 *	@exception DevFailed
 */
	Tango::DevLong	axis_get_motion_status();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_find_edge();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_find_index();
/**
 * 
 *	@exception DevFailed
 */
	void	axis_find_home();
/**
 * ethernet handle already in use
 *	@return	error code see command description
 *	@exception DevFailed
 */
	Tango::DevLong	axis_get_error_status();
/**
 * 
 *	@param	argin	
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevString	direct_cmd(Tango::DevString);
/**
 * 
 *	@exception DevFailed
 */
	void	axis_init();
/**
 * disable the power amplifier of the motor
 *	@exception DevFailed
 */
	void	motor_off();
/**
 * enables the power amplifier of the motor
 *	@exception DevFailed
 */
	void	motor_on();
/**
 * return an array of string
 *	describes in
 *	string 0 : the Axis letter
 *	string 1 : the motor config hardware and in properties
 *	string2 : the encoder config hard and in properties
 *	string 3 : the init type
 *	string 4 : the EncoderMotorRatio property
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevVarStringArray	*axis_read_hwconf();
/**
 * returns the UserEncodeRatio property,
 *	used to compute the gearing ratio in GalilSlit
 *	@return	
 *	@exception DevFailed
 */
	Tango::DevDouble	get_user_encoder_ratio();

/**
 *	Read the device properties from database
 */
	 void get_device_property();
//@}

		//	Here is the end of the automatic code generation part
		//------------------------------------------------------------- 
		
		
		
protected : 
	//	Add your own data members here
	//-----------------------------------------
	std::string axis_commas;			// contient les virgules qui vont bien pour l'axe
	std::string axis_letter;						// contient la lettre qui va bien pour l'axe
	std::string command_concat;		// pour la manipulation de chaines de caracteres
	Tango::DevLong axis_error;		// return code of error see AxisGetErrorStatus documentation
	short initrunning;

	// write important values as AC,DC,SP,MT,... to the hardware
	void write_important_values(void);

	// try to create a trhread to stop that fucking AxisInit polling
    
    omni_thread * thread_killer;
    friend class stop_poll_command;
	
    // finite state machine for initialisation
    void galil_axis_finite_state_machine();
    int StateMachineValue;
    int reqforinit;
    int stopinit;
    int moving_for_init;
	
    int LSForward;
    int LSBackward;
	
	// members to manage cached value of motion_status
	
	Tango::DevLong cached_value_axis_motion_status; 
	
	long		time_last_read_on_hw;
	bool	axis_motion_status_changed;	// used to avoid status recalculation
	bool	state_changed;	   // used to avoid status recalculation
       bool    status_changed;	   // used to avoid status recalculation

    // To read msec clock
    long msec_clock();
	
};

class stop_poll_command : public omni_thread, Tango::LogAdapter
{
public : 
    stop_poll_command(GalilAxis*);
private :
    GalilAxis *ga;
    void* run_undetached(void *arg);
};

} ;	// namespace

#endif	// _GALILAXIS_H
